"""
Author: RedFantom
License: GNU GPLv3
Copyright (c) 2017-2018 RedFantom
"""
import tkinter as tk
from ttkthemes import ThemedStyle, THEMES
import unittest


class TestThemedStyle(unittest.TestCase):
    def setUp(self):
        self.window = tk.Tk()

    def test_themed_style_init(self):
        ThemedStyle(self.window)

    def test_themed_style_themes(self):
        style = ThemedStyle(self.window)
        for item in THEMES:
            style.theme_use(item)
            self.assertTrue(item in style.themes)
            self.assertEqual(style.theme_use(), item)

    def test_custom_theme(self):
        if not ThemedStyle().png_support:
            return
        for theme in ThemedStyle.pixmap_themes:
            window = tk.Tk()
            style = ThemedStyle(window)
            style.set_theme_advanced(theme, brightness=0.2, saturation=1.3, hue=1.4)
            window.destroy()
        return



"""
Author: RedFantom
License: GNU GPLv3
Copyright (c) 2017-2018 RedFantom
"""
import sys
from unittest import TestCase
from ttkthemes import ThemedTk
from tkinter import ttk


def printf(string, end="\n"):
    sys.__stdout__.write(string + end)
    sys.__stdout__.flush()


class TestThemedWidgets(TestCase):
    """
    Tkinter may crash if widgets are not configured properly in a theme.
    Therefore, in addition to checking if all files for a theme exist
    by loading it, this Test also tests every core ttk widget to see
    if the widget can be successfully created with the theme data.

    When Tkinter crashes, it keeps the Global Interpreter Lock in place,
    so the program actually has to be terminated with SIGTERM.
    Therefore, this test only executes on UNIX.
    """
    WIDGETS = [
        "Label",
        "Treeview",
        "Button",
        "Frame",
        "Notebook",
        "Progressbar",
        "Scrollbar",
        "Scale",
        "Entry",
        "Combobox"
    ]

    def setUp(self):
        self.window = ThemedTk()

    def test_widget_creation(self):
        try:
            import signal
        except ImportError:
            pass
        if "signal" not in locals() or not hasattr(signal, "alarm"):
            return
        for theme in self.window.themes:
            self.window.set_theme(theme)
            for widget in self.WIDGETS:
                window = ThemedTk(theme=theme)
                signal.alarm(5)
                printf("Testing {}: {}".format(theme, widget), end=" - ")
                getattr(ttk, widget)(window).pack()
                window.update()
                window.destroy()
                signal.alarm(0)
                printf("SUCCESS")

    def tearDown(self):
        self.window.destroy()



"""
Author: RedFantom
License: GNU GPLv3
Copyright (c) 2017-2018 RedFantom
"""
from unittest import TestCase
from ttkthemes import _utils as utils
import os


class TestUtils(TestCase):
    def assertPathEquals(self, a, b):
        if hasattr(os.path, 'samefile'):
            self.assertTrue(os.path.samefile(a, b))
        else:
            # On windows, os.path.normcase lowercases because 'ASD' and 'asd'
            # should be treated equally
            self.assertEqual(os.path.normcase(a), os.path.normcase(b))

    def test_temporary_chdir(self):
        dir1 = os.getcwd()
        with utils.temporary_chdir(utils.get_temp_directory()):
            dir2 = os.getcwd()
        dir3 = os.getcwd()
        self.assertPathEquals(dir1, dir3)
        self.assertPathEquals(dir2, utils.get_temp_directory())

        with self.assertRaises(RuntimeError):
            with utils.temporary_chdir(utils.get_temp_directory()):
                raise RuntimeError()
        dir4 = os.getcwd()
        self.assertPathEquals(dir1, dir4)

    def test_get_file_directory(self):
        directory = utils.get_file_directory()
        self.assertIsInstance(directory, str)
        self.assertTrue(os.path.exists(directory))

    def test_get_temp_directory(self):
        self.assertTrue(os.path.exists(utils.get_temp_directory()))

    def test_get_themes_directory(self):
        themes = [
            "aquativo",
            "black",
            "blue",
            "clearlooks",
            "elegance",
            "keramik",
            "kroc",
            "plastik",
            "radiance",
            "winxpblue",
        ]
        folders = os.listdir(utils.get_themes_directory())
        for theme in themes:
            self.assertTrue(theme in folders)




"""
Author: RedFantom
License: GNU GPLv3
Copyright (c) 2017-2018 RedFantom
"""
import tkinter as tk
from tkinter import ttk
from ttkthemes import ThemedTk, THEMES
import unittest


class TestThemedTk(unittest.TestCase):

    def setUp(self):
        self.tk = ThemedTk()

    def tearDown(self):
        self.tk.destroy()

    def test_themes_available(self):
        available_themes = self.tk.get_themes()
        for theme in THEMES:
            if theme not in available_themes:
                raise AssertionError("Theme {} not available".format(theme))

    def test_theme_setting(self):
        button = ttk.Button(self.tk)
        label = ttk.Label(self.tk)
        button.pack()
        label.pack()
        self.tk.update()
        for theme in self.tk.get_themes():
            self.tk.set_theme(theme)
            self.tk.update()

    def test_custom_theme(self):
        if not self.tk.png_support:
            return
        for theme in self.tk.pixmap_themes:
            tk = ThemedTk()
            tk.set_theme_advanced(theme, brightness=0.2, saturation=1.4, hue=1.8)
            tk.destroy()
        return

    def test_toplevel_hook(self):
        __init__toplevel = tk.Toplevel.__init__
        self.tk.set_theme("black", True, False)
        self.assertNotEqual(__init__toplevel, tk.Toplevel.__init__)
        top = tk.Toplevel(self.tk)
        color = ttk.Style(self.tk).lookup("TFrame", "background")
        self.assertIsNotNone(color)
        self.assertEqual(top.cget("background"), color)
        top.destroy()

    def test_tk_background(self):
        self.tk.config(background="white")
        self.tk.set_theme("black", False, True)
        self.assertNotEqual(self.tk.cget("background"), "white")

    def test_config_cget(self):
        self.tk.config(theme="equilux")
        self.assertEqual(self.tk.cget("theme"), self.tk.current_theme)
        self.assertEqual(self.tk.cget("theme"), "equilux")

        self.tk.config(themebg=True)
        self.assertTrue(self.tk.cget("themebg"))
        before = self.tk.cget("bg")
        self.tk.config(themebg=False)
        self.assertFalse(self.tk.cget("themebg"))
        after = self.tk["bg"]
        self.assertNotEqual(before, after)

        self.tk.config(toplevel=False)
        self.assertFalse(self.tk.cget("toplevel"))
        orig = tk.Toplevel.__init__
        self.tk["toplevel"] = True
        self.assertTrue(self.tk.cget("toplevel"))
        self.assertNotEqual(orig, tk.Toplevel.__init__)

        self.tk.configure(toplevel=False)
        self.assertEqual(tk.Toplevel.__init__, orig)

    def test_background_on_init(self):
        w = ThemedTk(theme="black", toplevel=True)
        tk.Toplevel(w)  # Issue #74, RecursionError



"""
Author: RedFantom
License: GNU GPLv3
Copyright (c) 2017-2018 RedFantom

Simple script to convert GTK-theme SVG images to PNG images

Depends on inkscape
"""
import os
import subprocess
from ttkthemes._utils import get_themes_directory

print("** SVG to PNG converter **")

theme = input("Theme name: ")
png_path = os.path.join(get_themes_directory(theme, True), theme, theme)
if not os.path.exists(png_path):
    print("Please copy the SVG images to {} first.".format(png_path))
    exit(-1)

print("Converting SVG images to PNG...", end="", flush=True)
os.chdir(png_path)
for svg_image in os.listdir(os.getcwd()):
    if not svg_image.endswith((".svg", ".svg.in")):
        continue
    png_image = svg_image.replace(".svg", ".png").replace(".in", "")
    command = ["inkscape", "-z", "-e", png_image, svg_image]
    p = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    p.wait()
    stdout, stderr = p.communicate()
    if len(stderr) != 0 and not b"Format autodetect failed" in stderr:
        print("\nSubprocess error: {}".format(stderr))
        exit(-2)
    print(".", end="", flush=True)
    os.remove(svg_image)
print(" Done.")




"""
Author: RedFantom
License: GNU GPLv3
Copyright (c) 2017-2018 RedFantom

Simple script to convert a PNG-based theme to a GIF-based theme on Linux

Depends on imagemagick
"""
import os
from shutil import copytree, rmtree
import subprocess
from ttkthemes._utils import get_themes_directory


print("** PNG to GIF Theme Converter **\n")
theme = input("PNG-theme name: ")
png_path = os.path.join(get_themes_directory(theme, True), theme)
if not os.path.exists(png_path):
    print("Invalid theme name: Directory does not exist: {}".format(png_path))
    exit(-1)

gif_path = os.path.join(get_themes_directory(theme, False), theme)
if os.path.exists(gif_path):
    a = input("GIF-theme directory exists. Overwrite? (y/n) [y]: ",)
    if a != "n":
        print("Deleting everything under '{}'...".format(gif_path), end=" ")
        rmtree(gif_path)
        print("Done.")
    else:
        print("Aborted by user.")
        exit(0)


print("Copying 'png/{0}' to 'gif/{0}'...".format(theme), end=" ")
copytree(png_path, gif_path)
print("Done.")

print("Replacing PNG with GIF references...", end=" ")
tcl_file = os.path.join(gif_path, theme + ".tcl")
with open(tcl_file) as fi:
    lines = fi.readlines()
with open(tcl_file, "w") as fo:
    fo.writelines([line.replace("png", "gif") for line in lines])
print("Done.")

print("Converting all PNG images to GIF images...", end=" ", flush=True)
img_path = os.path.join(gif_path, theme)
os.chdir(img_path)
for image in os.listdir(os.getcwd()):
    if not image.endswith(".png"):
        continue
    command = ["convert", image, "-channel", "A", "-threshold", "50%", image.replace(".png", ".gif")]
    p = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    p.wait()
    stdout, stderr = p.communicate()
    if len(stderr) != 0:
        print("\nError in subprocess:\n{}".format(stderr))
        exit(-2)
    os.remove(image)
print("Done.")



"""
Author: RedFantom
License: GNU GPLv3
Copyright (c) 2017-2018 RedFantom
"""
import os
from shutil import rmtree
import sys

DEPENDENCIES = ["pillow"]
REQUIREMENTS = ["codecov", "coverage", "nose", "setuptools", "pip", "wheel", "semantic_version"]
PACKAGES = "python-tk python3-tk libtk-img"

SDIST = os.environ.get("SDIST", "false") == "true"

TO_DELETE = ["ttkthemes"]


class Version(object):
    """
    Parses a semantic version string.
    """
    def __init__(self, string):
        """
        :param string: semantic version string (major.minor.patch)
        """
        elements = tuple(map(int, string.split(".")))
        if len(elements) == 3:
            self.major, self.minor, self.patch = elements
        else:
            (self.major, self.minor), self.patch = elements, 0
        self.version = (self.major, self.minor, self.patch)

    def __ge__(self, other):
        return all(elem1 >= elem2 for elem1, elem2 in zip(self.version, other.version))


def run_command(command):
    """
    :param command: command to run on os.system
    :return: exit code
    """
    print("Running system command: ", command)
    return_info = os.system(command)
    if sys.platform == "win32":
        return return_info
    else:
        return os.WEXITSTATUS(return_info)


def check_wheel_existence():
    """Return True if a wheel is built"""
    return len([file for file in os.listdir("dist") if file.endswith((".whl", ".tar.gz"))]) != 0


def build_and_install_wheel(python):
    """Build a binary distribution wheel and install it"""
    dist_type = "bdist_wheel" if not SDIST else "sdist"
    return_code = run_command("{} setup.py {}".format(python, dist_type))
    if return_code != 0:
        print("Building and installing wheel failed.")
        exit(return_code)
    # Check if an artifact exists
    assert check_wheel_existence()
    print("Wheel file exists.")
    # Install the wheel file
    wheel = [file for file in os.listdir("dist") if file.endswith((".whl", ".tar.gz"))][0]
    wheel = os.path.join("dist", wheel)
    print("Wheel file:", wheel)
    return_code = run_command("{} -m pip install --ignore-installed {}".format(python, wheel))
    if return_code != 0:
        print("Installation of wheel failed.")
        exit(return_code)
    print("Wheel file installed.")


def ci(python="python", codecov="codecov", coverage_file="coverage.xml", wheel=True):
    """
    Run the most common CI tasks
    """

    # Import pip
    from pip import __version__ as pip_version
    if Version(pip_version) >= Version("10.0.0"):
        import pip._internal as pip
    else:
        import pip

    # Install requirements with pip
    pip.main(["install"] + DEPENDENCIES + REQUIREMENTS + ["-U"])
    # Build the installation wheel
    if wheel is True:
        build_and_install_wheel(python)
        # Remove all non-essential files
        for to_delete in TO_DELETE:
            rmtree(to_delete)
    # Run the tests on the installed ttkthemes
    return_code = run_command("{} -m nose --with-coverage --cover-xml --cover-package=ttkthemes".format(python))
    if return_code != 0:
        print("Tests failed.")
        exit(return_code)
    print("Tests successful.")
    # Run codecov
    return_code = run_command("{} -f {}".format(codecov, coverage_file))
    if return_code != 0:
        print("Codecov failed.")
        exit(return_code)
    # Successfully finished CI
    exit(0)


def ci_windows():
    """
    Run CI tasks on AppVeyor. CI on AppVeyor is relatively easy, so
    just the general ci() is used.
    """
    ci(
        python="%PYTHON%\\python.exe",
        codecov="%PYTHON%\\Scripts\\codecov.exe",
        coverage_file="C:\\projects\\ttk-themes\\coverage.xml",
        wheel=False
    )


def ci_linux():
    """
    Setup Travis-CI linux for installation and testing
    """
    run_command("sudo apt-get install {}".format(PACKAGES))
    ci()


# Run CI tasks on AppVeyor and Travis-CI (macOS and Linux)
if __name__ == '__main__':
    if sys.platform == "win32":
        ci_windows()
    elif "linux" in sys.platform:   # linux2 on Python 2, linux on Python 3
        ci_linux()
    else:
        raise RuntimeError("Invalid platform: ", sys.platform)




"""
Author: RedFantom
License: GNU GPLv3
Copyright (c) 2017-2018 RedFantom
"""
from unittest import TestCase
from ttkthemes import _imgops as imgops
from PIL import Image


class TestImgOps(TestCase):
    def test_make_transparent(self):
        transparent = imgops.make_transparent(Image.new("RGBA", (10, 10)))
        transparent_data = transparent.getdata()
        for i in range(0, 10*10):
            self.assertEqual(transparent_data[i], (255, 255, 255, 255))

    def test_shift_hue(self):
        red_image = Image.new("RGBA", (10, 10), (255, 0, 0, 255))
        shifted_image = imgops.shift_hue(red_image, 0.5)
        self.assertNotEqual(shifted_image.getdata()[0], red_image.getdata()[0])

    def test_check_pixel(self):
        black_image = Image.new("RGBA", (10, 10))
        for pixel in black_image.getdata():
            self.assertTrue(imgops._check_pixel(pixel))
        white_image = Image.new("RGBA", (10, 10), (255, 255, 255, 255))
        for pixel in white_image.getdata():
            self.assertFalse(imgops._check_pixel(pixel))



"""
Author: RedFantom
License: GNU GPLv3
Copyright (c) 2018 RedFantom
"""
import os
import tkinter as tk
from tkinter import ttk
from ttkthemes import ThemedTk, THEMES
from ttkwidgets import ScaleEntry
from ttkwidgets.autocomplete import AutocompleteCombobox
from PIL import Image


class Example(ThemedTk):
    """
    Example that is used to create screenshots for new themes.
    """
    def __init__(self, theme="arc"):
        """
        :param theme: Theme to show off
        """
        ThemedTk.__init__(self, fonts=True, themebg=True)
        self.set_theme(theme)
        # Create widgets
        self.notebook = ttk.Notebook(self)
        self.notebook.add(ttk.Button(self, text="Hello World"), text="Frame One")
        self.notebook.add(ttk.Button(self, text="Hello Universe"), text="Frame Two")
        self.menu = tk.Menu(self, tearoff=False)
        self.sub_menu = tk.Menu(self.menu, tearoff=False)
        self.sub_menu.add_command(label="Exit", command=self.destroy)
        self.menu.add_cascade(menu=self.sub_menu, label="General")
        self.config(menu=self.menu)
        self.label = ttk.Label(self, text="This is an example label.")
        self.dropdown = ttk.OptionMenu(self, tk.StringVar(), "First value", "Second Value")
        self.entry = ttk.Entry(self, textvariable=tk.StringVar(value="Default entry value."))
        self.button = ttk.Button(self, text="Button")
        self.radio_one = ttk.Radiobutton(self, text="Radio one", value=True)
        self.radio_two = ttk.Radiobutton(self, text="Radio two", value=False)
        self.scroll = ttk.Scrollbar(self, orient=tk.VERTICAL)
        self.checked = ttk.Checkbutton(self, text="Checked", variable=tk.BooleanVar(value=True))
        self.unchecked = ttk.Checkbutton(self, text="Unchecked")
        self.tree = ttk.Treeview(self, height=4, show=("tree", "headings"))
        self.setup_tree()
        self.scale_entry = ScaleEntry(self, from_=0, to=50, orient=tk.HORIZONTAL, compound=tk.RIGHT)
        self.combo = AutocompleteCombobox(self, completevalues=["something", "something else"])
        self.progress = ttk.Progressbar(self, maximum=100, value=50)
        # Grid widgets
        self.grid_widgets()
        # Bind screenshot button
        self.bind("<F10>", self.screenshot)
        self.bind("<F9>", self.screenshot_themes)

    def setup_tree(self):
        """Setup an example Treeview"""
        self.tree.insert("", tk.END, text="Example 1", iid="1")
        self.tree.insert("", tk.END, text="Example 2", iid="2")
        self.tree.insert("2", tk.END, text="Example Child")
        self.tree.heading("#0", text="Example heading")

    def grid_widgets(self):
        """Put widgets in the grid"""
        sticky = {"sticky": "nswe"}
        self.notebook.grid(row=0, column=1, columnspan=2, **sticky)
        self.label.grid(row=1, column=1, columnspan=2, **sticky)
        self.dropdown.grid(row=2, column=1, **sticky)
        self.entry.grid(row=2, column=2, **sticky)
        self.button.grid(row=3, column=1, columnspan=2, **sticky)
        self.radio_one.grid(row=4, column=1, **sticky)
        self.radio_two.grid(row=4, column=2, **sticky)
        self.checked.grid(row=5, column=1, **sticky)
        self.unchecked.grid(row=5, column=2, **sticky)
        self.scroll.grid(row=1, column=3, rowspan=8, padx=5, **sticky)
        self.tree.grid(row=6, column=1, columnspan=2, **sticky)
        self.scale_entry.grid(row=7, column=1, columnspan=2, **sticky)
        self.combo.grid(row=8, column=1, columnspan=2, **sticky)
        self.progress.grid(row=9, column=1, columnspan=2, padx=5, pady=5, **sticky)

    def screenshot(self, *args):
        """Take a screenshot, crop and save"""
        from mss import mss
        if not os.path.exists("screenshots"):
            os.makedirs("screenshots")
        box = {
            "top": self.winfo_y(),
            "left": self.winfo_x(),
            "width": self.winfo_width(),
            "height": self.winfo_height()
        }
        screenshot = mss().grab(box)
        screenshot = Image.frombytes("RGB", screenshot.size, screenshot.rgb)
        screenshot.save("screenshots/{}.png".format(ttk.Style(self).theme_use()))

    def screenshot_themes(self, *args):
        """Take a screenshot for all themes available"""
        from time import sleep
        for theme in THEMES:
            self.set_theme(theme)
            self.update()
            sleep(0.05)
            self.screenshot()


if __name__ == '__main__':
    example = Example()
    example.set_theme("adapta")
    example.mainloop()



"""
Author: RedFantom
License: GNU GPLv3
Copyright (c) 2017-2018 RedFantom
"""
import os
from tkinter import TkVersion
from setuptools import setup


if TkVersion <= 8.5:
    message = "This version of ttkthemes does not support Tk 8.5 and earlier. Please install an earlier version."
    raise RuntimeError(message)


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name="ttkthemes",
    packages=["ttkthemes"],
    package_data={"ttkthemes": ["themes/*", "png/*", "gif/*", "advanced/*"]},
    version="3.2.0",
    description="A group of themes for the ttk extensions of Tkinter with a Tkinter.Tk wrapper",
    author="The ttkthemes authors",
    author_email="redfantom@outlook.com",
    url="https://github.com/RedFantom/ttkthemes",
    download_url="https://github.com/RedFantom/ttkthemes/releases",
    include_package_data=True,
    keywords=["tkinter", "ttk", "gui", "tcl", "theme"],
    license="GPLv3",
    long_description=read("README.md"),
    classifiers=[
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Environment :: Win32 (MS Windows)",
        "Environment :: X11 Applications",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Topic :: Software Development :: Libraries :: Tcl Extensions",
        "Topic :: Software Development :: Libraries :: Python Modules"
    ],
    zip_safe=False,
    install_requires=["pillow"],
    has_ext_modules=lambda: True,
    python_requires=">=3.5"
)
